﻿local _G = _G
local select = _G.select
local pairs = _G.pairs
local ipairs = _G.ipairs
local string = _G.string
local type = _G.type
local error = _G.error
local table = _G.table
local CreateFrame = _G.CreateFrame


local codex_data = {
	player = { },
	location = { },
}

local function helper_CodexInit( loc_id )
	
	if not codex_data.location[loc_id] then
		codex_data.location[loc_id] = { }
	end
	
	local codex = codex_data.location[loc_id]
	
	codex.loc_id = loc_id
	
	if not codex.player then
		codex.player = { }
	end
	
	if not codex.workpad then
		codex.workpad = { }
	end
	
	if not codex.global then
		codex.global = ArkInventory.acedb.global.options
	end
	
	return codex
	
end

function ArkInventory.GetPlayerStorage( loc_id, player_id, player )
	
	local loc_id = loc_id or ArkInventory.Const.Location.Bag
	local player_id = player_id or ArkInventory.Global.Me.data.info.player_id
	local player = player or { }
	
	player.data = ArkInventory.db.player.data[player_id]
	
	if ( loc_id == ArkInventory.Const.Location.Vault ) then
		player_id = player.data.info.guild_id
	elseif ( loc_id == ArkInventory.Const.Location.Pet ) or ( loc_id == ArkInventory.Const.Location.Mount ) or ( loc_id == ArkInventory.Const.Location.Toybox ) or ( loc_id == ArkInventory.Const.Location.Heirloom ) then
		player_id = ArkInventory.PlayerIDAccount( )
		player.data = ArkInventory.db.player.data[player_id]
	end
	
	if player_id then
		player.data = ArkInventory.db.player.data[player_id]
	end
	
	return player
	
end

function ArkInventory.GetPlayerInfo( player_id )
	local player_id = player_id or ArkInventory.Global.Me.data.info.player_id
	return ArkInventory.db.player.data[player_id].info
end

function ArkInventory.GetPlayerCodex( loc_id, rebuild )
	
	--debugprofilestart()
	
	local loc_id = loc_id or ArkInventory.Const.Location.Bag
	
	if not codex_data.player[loc_id] then
		codex_data.player[loc_id] = { }
	end
	
	local codex = codex_data.player[loc_id]
	
	if not codex.player then
		codex.player = { }
	end
	
	if not codex.workpad then
		codex.workpad = { }
	end
	
	if not codex.global then
		codex.global = ArkInventory.acedb.global.options
	end
	
	local player_id = ArkInventory.Global.Me.data.info.player_id
	
	if not rebuild and codex.player.current then
		-- nothing has changed with settings
		--print("cached player codex for "..player_id.." / "..loc_id)
		return codex, false
	end
	
	codex.player.previous = player_id
	codex.player.current = player_id
	
	ArkInventory.GetPlayerStorage( nil, player_id, codex.player )
	
	local old_id, changed
	
	old_id = codex.profile_id
	codex.profile_id, codex.profile = ArkInventory.ConfigInternalProfileGet( codex.player.data.profile, true )
	if codex.profile_id ~= old_id then
		changed = true
	end
	
	old_id = codex.style_id
	codex.style_id, codex.style = ArkInventory.ConfigInternalDesignGet( codex.profile.location[loc_id].style, true )
	if codex.style_id ~= old_id then
		changed = true
	end

	old_id = codex.layout_id
	codex.layout_id, codex.layout = ArkInventory.ConfigInternalDesignGet( codex.profile.location[loc_id].layout, true )
	if codex.layout_id ~= old_id then
		changed = true
	end
	
	old_id = codex.catset_id
	codex.catset_id, codex.catset = ArkInventory.ConfigInternalCategorysetGet( codex.profile.location[loc_id].catset, true )
	if codex.catset_id ~= old_id then
		changed = true
	end
	
	-- account data is set last to ensure that player_id above was for the user, not the account/vault
	ArkInventory.GetPlayerStorage( loc_id, player_id, codex.player )
	
	if changed then
		ArkInventory.Frame_Main_DrawStatus( nil, ArkInventory.Const.Window.Draw.Init )
	end
	
	
	--local ms = debugprofilestop()
	--print("built player codex for "..player_id.." / "..loc_id.." in "..ms)
	
	return codex, changed

end

function ArkInventory.GetLocationCodex( loc_id, rebuild )
	
	--debugprofilestart()
	
	local codex = helper_CodexInit( loc_id )
	
	if not rebuild and codex.player.current and codex.player.previous and codex.player.current == codex.player.previous then
		-- same character as last time, and not nil, and nothing has changed with settings
		return codex, false
	end
	
	local player_id = codex.player.current or ArkInventory.Global.Me.data.info.player_id
	codex.player.current = player_id
	codex.player.previous = player_id
	
	ArkInventory.GetPlayerStorage( nil, player_id, codex.player )
	
	local old_id, changed
	
	old_id = codex.profile_id
	codex.profile_id, codex.profile = ArkInventory.ConfigInternalProfileGet( codex.player.data.profile, true )
	if codex.profile_id ~= old_id then
		changed = true
	end
	
	old_id = codex.style_id
	codex.style_id, codex.style = ArkInventory.ConfigInternalDesignGet( codex.profile.location[loc_id].style, true )
	if codex.style_id ~= old_id then
		changed = true
	end

	old_id = codex.layout_id
	codex.layout_id, codex.layout = ArkInventory.ConfigInternalDesignGet( codex.profile.location[loc_id].layout, true )
	if codex.layout_id ~= old_id then
		changed = true
	end
	
	old_id = codex.catset_id
	codex.catset_id, codex.catset = ArkInventory.ConfigInternalCategorysetGet( codex.profile.location[loc_id].catset, true )
	if codex.catset_id ~= old_id then
		changed = true
	end
	
	-- account data is set last to ensure that player_id above was for the user, not the account/vault
	ArkInventory.GetPlayerStorage( loc_id, player_id, codex.player )
	
	if changed then
		ArkInventory.Frame_Main_DrawStatus( nil, ArkInventory.Const.Window.Draw.Init )
	end
	
	
	--local ms = debugprofilestop()
	--print("built location codex for "..player_id.." / "..loc_id.." in "..ms)

	return codex, changed
	
end

function ArkInventory.SetLocationCodex( loc_id, player_id )
	
	local codex = helper_CodexInit( loc_id )
	
	local player = ArkInventory.GetPlayerStorage( loc_id, player_id )
	
	local player_id = player.data.info.player_id
	
	codex.player.previous = codex.player.current
	codex.player.current = player_id
	
	return ArkInventory.GetLocationCodex( loc_id )
	
end

function ArkInventory.CodexReset( loc_id )
	
	if not loc_id then
		table.wipe( codex_data.player )
		table.wipe( codex_data.location )
	else
		codex_data.player[loc_id] = nil
		codex_data.location[loc_id] = nil
	end
	
end
